import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ModalData {
  final BuildContext context;
  final String text;
  final double width;
  final bool maskClosable;
  final String okText;
  final String cancelText;
  final void Function () onOk;
  final void Function () onCancel;

  ModalData({
    @required this.context,
    @required this.text,
    this.width = 300.0,
    this.maskClosable,
    this.okText,
    this.cancelText,
    this.onOk,
    this.onCancel
  });

  factory ModalData.fromJson(Map<String, dynamic> json) {
    return ModalData(
      context: json['context'],
      width: json['width'] ?? 300.0,
      maskClosable: json['maskClosable'] ?? true,
      text: json['text'],
      onOk: json['onOk'],
      onCancel: json['onCancel']
    );
  }
}

class ModalContent extends StatelessWidget {

  final double width;
  final String text;
  final String okText;
  final String cancelText;
  final void Function () onOk;
  final void Function () onCancel;

  ModalContent({
    this.width = 300.0,
    this.text,
    this.okText = 'Ok',
    this.cancelText = 'Cancel',
    this.onOk,
    this.onCancel
  });

  TextStyle _initialStyle(
    [double size = 15.0,
    bool bold = false,
    Color color = Colors.black]) {
    return TextStyle(
      fontSize: size,
      color: color,
      fontWeight: bold ? FontWeight.w500 : FontWeight.normal,
      decoration: TextDecoration.none
    );
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Container(
          width: width,
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.circular(10),
          ),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Container(
                padding: EdgeInsets.symmetric(vertical: 20.0),
                child: Text(text, style: _initialStyle(16.0, true, Color(0xff363636)))
              ),
              Container(
                decoration: BoxDecoration(
                  border: Border(top: BorderSide(width: 1.0, color: Color(0xffeeeeee)))
                ),
                child: Row(children: <Widget>[
                  OwnButton(
                    onClick: () {
                      if (onCancel != null) {
                        onCancel();
                      }
                      Navigator.pop(context);
                    },
                    hasBorder: true,
                    text: Text(
                      cancelText,
                      style: _initialStyle(15.0, false, Color(0xff676767))
                    )
                  ),
                  OwnButton(
                    onClick: () {
                      if (onOk != null) {
                        onOk();
                      }
                      Navigator.pop(context);
                    },
                    text: Text(
                      okText,
                      style: _initialStyle(15.0, false, Color(0xfffc4c63))
                    )
                  )
                ])
              )
            ]
          )
        )
      ]
    );
  }
}

class OwnButton extends StatelessWidget {

  final void Function () onClick;
  final bool hasBorder;
  final Text text;

  OwnButton({
    this.onClick,
    this.hasBorder = false,
    this.text
  });

  @override
  Widget build(BuildContext context) {
    return Expanded(child: GestureDetector(
      onTap: () {
        if (onClick != null) {
          onClick();
        }
      },
      child: Container(
        padding: EdgeInsets.symmetric(vertical: 12.0),
        decoration: BoxDecoration(
          border: Border(
            right: BorderSide(
              width: hasBorder ? 1.0 : 0,
              color: Color(0xffeeeeee)
            )
          )
        ),
        child: Center(child: text)
      ))
    );
  }
}

class Modal {
  static show(ModalData data) {
    return showDialog<Null>(
      context: data.context,
      barrierDismissible: data.maskClosable ?? true,
      builder: (context) {
        return ModalContent(
          width: data.width,
          text: data.text,
          okText: data.okText,
          cancelText: data.cancelText,
          onOk: data.onOk,
          onCancel: data.onCancel
        );
      }
    );
  }
}